schema = {
    "type": "object",
    "properties": {
        "rcn": {
            "type": "object",
            "properties": {
                "profile": {"type": "string" },
                "binders": {
                    "type": "object",
                    "properties": {
                        "message": {
                            "type": "object"
                        }
                    }
                }
            }
        }
    }
}