RCN_EVENTS_ENABLE = "rcn.events.enable"
RCN_EVENTS_NUM_THREADS = "rcn.events.num.threads"

RCN_FLASK_APP_ENABLED = "app.flask.enabled"
RCN_FLASK_APP_NAME = "app.flask.name"
RCN_FLASK_APP_PORT = "app.flask.port"
RCN_FLASK_APP_LISTEN = "app.flask.listen"
RCN_FLASK_APP_CONTEXT_URL = "app.flask.contextUrl"
RCN_FLASK_APP_CONFIGS_LIST = "app.flask.configs"
RCN_FLASK_APP_BACKEND_SERVER = "app.flask.backend"

RCN_AUTO_CONFIGURE_SECRETS_ENABLE = "app.autoconfiguration.secrets.enable"
RCN_ENABLE_VAULT_PROVIDER = "secrets.provider.vault.enable"

RCN_ENABLE_STANDALONE_MESSAGE_BROKER = "rcn.message.brokers.standalone.enable"
RCN_ENABLE_CONTEXT_THREADS = "management.context.thread.enable"

RCN_ENABLE_TASK_EXECUTOR = "task.executor.enable"
RCN_TASK_EXECUTOR_NUM_THREADS = "task.executor.num.threads"